# -*- coding: utf-8 -*-
# @Time    : 6/4/2021 7:51 AM
# @Author  : Paulo Radatz
# @Email   : paulo.radatz@gmail.com
# @File    : test_circuit.py
# @Software: PyCharm

import platform
import pytest


class TestCircuit13Bus:

    @pytest.fixture(scope='function')
    def dss(self, solve_snap_13bus):
        return solve_snap_13bus

    def test_num_ckt_elements(self, dss):
        expected = 39
        actual = dss.circuit.num_ckt_elements
        assert actual == expected

    def test_num_buses(self, dss):
        expected = 16
        actual = dss.circuit.num_buses
        assert actual == expected

    def test_num_nodes(self, dss):
        expected = 41
        actual = dss.circuit.num_nodes
        assert actual == expected

    def test_first_pc_element(self, dss):
        expected = 1
        actual = dss.circuit.pc_element_first()
        assert actual == expected

    def test_next_pc_element(self, dss):
        expected = 3
        dss.circuit.pc_element_first()
        dss.circuit.pc_element_next()
        actual = dss.circuit.pc_element_next()
        assert actual == expected

    def test_first_pd_element(self, dss):
        expected = 1
        actual = dss.circuit.pd_element_first()
        assert actual == expected

    def test_next_pd_element(self, dss):
        expected = 3
        dss.circuit.pd_element_first()
        dss.circuit.pd_element_next()
        actual = dss.circuit.pd_element_next()
        assert actual == expected

    def test_sample(self, dss):
        expected = 0
        actual = dss.circuit.sample
        assert actual == expected

    def test_save_sample(self, dss):
        expected = 0
        actual = dss.circuit.save_sample
        assert actual == expected

    def test_set_active_bus_i(self, dss):
        expected = 0
        actual = dss.circuit.set_active_bus_i(1)
        assert actual == expected

    def test_first_element(self, dss):
        expected = 1
        actual = dss.circuit.first_element()
        assert actual == expected

    def test_next_element(self, dss):
        expected = 2
        dss.circuit.first_element()
        actual = dss.circuit.next_element()
        assert actual == expected

    def test_update_storage_t(self, dss):
        expected = 0
        actual = dss.circuit.update_storage_t
        assert actual == expected

    def test_parent_pd_element(self, dss):
        expected = 1
        dss.circuit.first_element()
        dss.circuit.next_element()
        actual = dss.circuit.parent_pd_element
        assert actual == expected

    def test_end_of_time_step_update(self, dss):
        expected = 0
        actual = dss.circuit.end_of_time_step_update
        assert actual == expected

    def test_name(self, dss):
        expected = 'ieee13nodeckt'
        actual = dss.circuit.name
        assert actual == expected

    def test_disable(self, dss):
        pass

    def test_enable(self, dss):
        pass

    def test_activate_element(self, dss):
        expected = '26'
        actual = dss.circuit.set_active_element('Line.650632')
        assert actual == expected

    def test_activate_class_get_name(self, dss):
        expected = '650632'
        dss.circuit.set_active_element("Line.650632")
        actual = dss.active_class.name
        assert actual == expected

    def test_set_activate_bus(self, dss):
        expected = '8'
        actual = dss.circuit.set_active_bus("692")
        assert actual == expected

    def test_set_activate_class(self, dss):
        # Not sure how to use it
        expected = '23'
        actual = dss.circuit.set_active_class("Capacitor")
        assert actual == expected

    def test_losses(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [112405.24721569585, 327901.77539540455]
            actual = dss.circuit.losses
            assert [round(value, 21) for value in actual] == [round(value, 21) for value in expected]

    def test_line_losses(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [106.49777151287977, 317.2152703226103]
            actual = dss.circuit.line_losses
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_substation_losses(self, dss):
        expected = [0.0, 0.0]
        actual = dss.circuit.substation_losses
        assert actual == expected

    def test_total_power(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [-3567.2118131482466, -1736.5765097263468]
            actual = dss.circuit.total_power
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_bus_volts(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [57502.68622482564,
                        33189.47561163729,
                        -10.988868660972098,
                        -66394.86888242468,
                        -57491.697356164674,
                        33205.39328374589,
                        2401.562774320432,
                        -0.46690091155609226,
                        -1201.2376790663666,
                        -2079.7175126796437,
                        -1200.3116016198273,
                        2080.1419403490627,
                        2536.356120228099,
                        -0.5793286234950956,
                        -1246.2598772664523,
                        -2157.4877137350218,
                        -1267.5877694598496,
                        2196.935539327288,
                        2426.4262832082777,
                        -109.96557825373392,
                        -1300.0178453618119,
                        -2096.2860825922003,
                        -1120.4252829545628,
                        2128.604896193387,
                        273.12085209243287,
                        -15.65326869615496,
                        -149.22087189563993,
                        -236.28815213487715,
                        -124.73890495654827,
                        242.00752558006553,
                        2350.078784828011,
                        -221.0796482325248,
                        -1338.403130602017,
                        -2109.8005657418066,
                        -1015.4071554561633,
                        2083.115730276637,
                        -1295.6847720988399,
                        -2078.3683079842394,
                        -1122.3895089817488,
                        2129.5620039280107,
                        -1296.2419078314438,
                        -2073.161856729391,
                        -1121.7911317832359,
                        2124.3537662394724,
                        -1015.4071496666263,
                        2083.115713199055,
                        2350.0787629180386,
                        -221.07964093085243,
                        -1338.4031344410694,
                        -2109.8005600677147,
                        2333.500398562236,
                        -229.7550420165211,
                        -1347.9807493549972,
                        -2110.4135775719074,
                        -1013.9682244842037,
                        2078.6483630432763,
                        -1002.1266180672632,
                        2078.753428367298,
                        2332.4291296649008,
                        -217.30934684180974,
                        2407.055227539429,
                        -145.37221180195039,
                        -1312.300335059707,
                        -2102.3729377007444,
                        -1083.00064496635,
                        2116.1077231577606,
                        2433.8502466486893,
                        -107.52285925941754,
                        -1300.7615949301178,
                        -2101.270878230694,
                        -1123.5656240409871,
                        2134.1389201731345,
                        2350.078813428106,
                        -221.07966574213089,
                        -1338.4031538526617,
                        -2109.800593205293,
                        -1015.407159783762,
                        2083.1157650090286,
                        2345.389845277566,
                        -221.599795561614,
                        -1009.5693326805186,
                        2080.532633420145]
            actual = dss.circuit.buses_volts
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_bus_vmag(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [66393.52539552496,
                        66394.86979179642,
                        66391.96794809647,
                        2401.562819706807,
                        2401.7070791741353,
                        2401.611632421512,
                        2536.356186390269,
                        2491.569167533136,
                        2536.3959701204935,
                        2428.9168236571245,
                        2466.670172181299,
                        2405.4753415452574,
                        273.56904917875966,
                        279.4619105506327,
                        272.26354300042607,
                        2360.454724361668,
                        2498.515832891544,
                        2317.417277288996,
                        2449.165950335357,
                        2407.2375326182864,
                        2445.0446146882796,
                        2402.3518200886697,
                        2317.4172594012734,
                        2360.454701864133,
                        2498.5158301567194,
                        2344.7838897054307,
                        2504.1760259677267,
                        2312.7711900328877,
                        2307.6987616644697,
                        2342.530468773924,
                        2411.4410522318562,
                        2478.326882914011,
                        2377.1416224123986,
                        2436.2241662817496,
                        2471.2992596093127,
                        2411.835119182977,
                        2360.4547544759903,
                        2498.5158685371916,
                        2317.417310405978,
                        2355.8353074279353,
                        2312.5410863842294]
            actual = dss.circuit.buses_vmag
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_element_names(self, dss):
        expected = ['Vsource.source', 'Transformer.sub', 'Transformer.reg1', 'RegControl.reg1', 'Transformer.reg2',
                    'RegControl.reg2', 'Transformer.reg3', 'RegControl.reg3', 'Transformer.xfm1', 'Load.671',
                    'Load.634a', 'Load.634b', 'Load.634c', 'Load.645', 'Load.646', 'Load.692', 'Load.675a', 'Load.675b',
                    'Load.675c', 'Load.611', 'Load.652', 'Load.670a', 'Load.670b', 'Load.670c', 'Capacitor.cap1',
                    'Capacitor.cap2', 'Line.650632', 'Line.632670', 'Line.670671', 'Line.671680', 'Line.632633',
                    'Line.632645', 'Line.645646', 'Line.692675', 'Line.671684', 'Line.684611', 'Line.684652',
                    'EnergyMeter.em1', 'Line.671692']
        actual = dss.circuit.elements_names
        assert actual == expected

    def test_all_bus_names(self, dss):
        expected = ['sourcebus', '650', 'rg60', '633', '634', '671', '645', '646', '692', '675', '611', '652', '670',
                    '632', '680', '684']
        actual = dss.circuit.buses_names
        assert actual == expected

    def test_all_element_losses(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [-3567.2118131482466,
                        -1736.5765097263468,
                        0.032287805258994925,
                        0.26246873644273727,
                        0.12209388323919848,
                        0.12385831108468119,
                        0.0,
                        0.0,
                        0.06534575459547341,
                        0.06707771651400253,
                        0.0,
                        0.0,
                        0.13508947523473763,
                        0.13685396666557062,
                        0.0,
                        0.0,
                        5.552658784487693,
                        10.096246342087282,
                        1155.0138982861056,
                        660.0382376763034,
                        160.00037738764144,
                        110.00734172751,
                        119.99921586065129,
                        89.99779381448776,
                        120.00422631732324,
                        90.00659913976904,
                        169.99904086275637,
                        124.99723393693854,
                        234.56787958279554,
                        134.62671049947636,
                        166.6740314624779,
                        148.05321488529,
                        485.0036235679137,
                        190.0283200695574,
                        67.9994060234033,
                        59.99809614195056,
                        290.01455466662827,
                        212.02260131243497,
                        163.45974879842845,
                        76.92806636588513,
                        121.93682536877358,
                        81.93452983101074,
                        16.999947095957086,
                        10.00069886182683,
                        65.99949909005798,
                        37.99886582656774,
                        117.00423402328067,
                        68.00542694061346,
                        -2.910383e-14,
                        -593.4894936633154,
                        0.0,
                        -92.45613844770358,
                        60.735771865096176,
                        196.00964718505844,
                        12.990226677136263,
                        41.49321184464521,
                        22.728048743811087,
                        72.33188446746534,
                        1.617935364e-11,
                        -0.004169129936205269,
                        0.8244626383287541,
                        1.056110348104019,
                        2.7672874779250125,
                        2.4007086567264633,
                        0.5274717697907181,
                        0.4197353422580782,
                        4.1628182200738815,
                        2.4192605657831883,
                        0.5794728640637477,
                        0.47066851977069746,
                        0.38239410523098194,
                        0.38733871407359405,
                        0.7998080968319555,
                        0.2308738086613739,
                        0.0,
                        0.0,
                        9.05457499902695e-06,
                        5.820766e-14]
            actual = dss.circuit.elements_losses
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_bus_vmag_pu(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [0.9999735589449024,
                        0.9999938073157516,
                        0.9999501017442392,
                        0.9999107743510974,
                        0.9999708379873679,
                        0.9999310979332906,
                        1.0560331204133613,
                        1.0373856703700681,
                        1.0560496847023189,
                        1.0112998427723447,
                        1.027018765320356,
                        1.0015397855554047,
                        0.9871572761581677,
                        1.0084213080290931,
                        0.9824464365113577,
                        0.9827950748943266,
                        1.040277972616208,
                        0.9648760737025178,
                        1.0197307360934018,
                        1.0022734885532727,
                        1.018014783512685,
                        1.000239281261813,
                        0.9648760662548149,
                        0.9827950655272896,
                        1.0402779714775408,
                        0.9762703917641309,
                        1.042634641363467,
                        0.962941636398677,
                        0.9608296884054429,
                        0.9753321611044653,
                        1.004023659116089,
                        1.031872134367998,
                        0.9897428045204293,
                        1.0143423160161344,
                        1.0289461246035285,
                        1.0041877321884265,
                        0.9827950874326766,
                        1.0402779874575714,
                        0.9648760874910502,
                        0.9808717420023628,
                        0.9628458308192339]
            actual = dss.circuit.buses_vmag_pu
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_node_names(self, dss):
        expected = ['sourcebus.1', 'sourcebus.2', 'sourcebus.3', '650.1', '650.2', '650.3', 'rg60.1', 'rg60.2',
                    'rg60.3', '633.1', '633.2', '633.3', '634.1', '634.2', '634.3', '671.1', '671.2', '671.3', '645.2',
                    '645.3', '646.2', '646.3', '692.3', '692.1', '692.2', '675.1', '675.2', '675.3', '611.3', '652.1',
                    '670.1', '670.2', '670.3', '632.1', '632.2', '632.3', '680.1', '680.2', '680.3', '684.1', '684.3']
        actual = dss.circuit.nodes_names
        assert actual == expected

    def test_system_y(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [0.8178822742754986,
                        -4.636999108384914,
                        -0.13055025449836521,
                        1.4833269062112926,
                        -0.13055025449836524,
                        1.4833269062112926,
                        -9.283352668986776,
                        74.26682135189421,
                        9.283352668986776,
                        -74.26682135189421,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -0.13055025449836521,
                        1.4833269062112926,
                        0.8178822742754985,
                        -4.636999108384914,
                        -0.13055025449836524,
                        1.4833269062112926,
                        0.0,
                        0.0,
                        -9.283352668986776,
                        74.26682135189421,
                        9.283352668986776,
                        -74.26682135189421,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -0.13055025449836524,
                        1.4833269062112926,
                        -0.13055025449836524,
                        1.4833269062112926,
                        0.8178822742754985,
                        -4.636999108384913,
                        9.283352668986776,
                        -74.26682135189421,
                        0.0,
                        0.0,
                        -9.283352668986776,
                        74.26682135189421,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -9.283352668986776,
                        74.26682135189421,
                        0.0,
                        0.0,
                        9.283352668986776,
                        -74.26682135189421,
                        1890.6787348910132,
                        -5002.165990528294,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1369.165023011177,
                        1369.165023011177,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        9.283352668986776,
                        -74.26682135189421,
                        -9.283352668986776,
                        74.26682135189421,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1890.6787348910132,
                        -5002.165990528294,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1393.9089692101743,
                        1393.9089692101743,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        9.283352668986776,
                        -74.26682135189421,
                        -9.283352668986776,
                        74.26682135189421,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1890.6787348910132,
                        -5002.165990528294,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1369.165023011177,
                        1369.165023011177,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1369.165023011177,
                        1369.165023011177,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1297.3960314120725,
                        -1299.5515100351274,
                        -0.48585754602498915,
                        1.22029123063551,
                        -0.2660589098474314,
                        0.912216808259467,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1.1451220523783032,
                        3.300600730734862,
                        0.48585754602498915,
                        -1.2202912734754099,
                        0.2660589098474314,
                        -0.9122168510993669,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1393.9089692101743,
                        1393.9089692101743,
                        0.0,
                        0.0,
                        -0.48585754602498915,
                        1.22029123063551,
                        1344.5294284920049,
                        -1346.6543624309695,
                        -0.12630483115283336,
                        0.6967002366036674,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.48585754602498915,
                        -1.2202912734754099,
                        -1.0027111809933547,
                        3.1276451752596324,
                        0.12630483115283336,
                        -0.6967002794435673,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1369.165023011177,
                        1369.165023011177,
                        -0.2660589098474314,
                        0.912216808259467,
                        -0.12630483115283336,
                        0.6967002366036674,
                        1297.1376277955421,
                        -1299.201503062629,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.2660589098474314,
                        -0.9122168510993669,
                        0.12630483115283336,
                        -0.6967002794435673,
                        -0.8867184358481344,
                        2.9505937582362023,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        6.174634422355675,
                        -8.24293119603154,
                        -1.495155595100418,
                        1.5059091102776052,
                        -2.1394799752143716,
                        1.6901170943579362,
                        -5.286769772134453,
                        9.612308676608098,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -5.564622525570931,
                        7.133818691956606,
                        1.495155595100418,
                        -1.5059091209875801,
                        2.1394799752143716,
                        -1.6901171050679111,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1.495155595100418,
                        1.5059091102776052,
                        5.5185532759822955,
                        -8.172535006552001,
                        -1.0880240059003325,
                        1.3823966977507467,
                        0.0,
                        0.0,
                        -5.286769772134453,
                        9.612308676608098,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1.495155595100418,
                        -1.5059091209875801,
                        -4.908541379197551,
                        7.063422502477067,
                        1.0880240059003325,
                        -1.3823967084607216,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -2.1394799752143716,
                        1.6901170943579362,
                        -1.0880240059003325,
                        1.3823966977507467,
                        5.799215580883693,
                        -8.194725900906395,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -5.286769772134453,
                        9.612308676608098,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.1394799752143716,
                        -1.6901171050679111,
                        1.0880240059003325,
                        -1.3823967084607216,
                        -5.189203684098949,
                        7.085613396831461,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -5.286769772134453,
                        9.612308676608098,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        47.90393247228869,
                        -84.7402932980703,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -5.286769772134453,
                        9.612308676608098,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        47.38261719384117,
                        -84.47963565884653,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -5.286769772134453,
                        9.612308676608098,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        47.38261719384117,
                        -84.47963565884653,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        10000011.227919342,
                        -18.254187896137672,
                        -1.7229307871034958,
                        4.284189778778654,
                        -3.3612866353936672,
                        4.205156433963181,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -10000000.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1.718112606719134,
                        4.952139130885014,
                        0.7289685611777779,
                        -1.8308946338715828,
                        0.3991881618116,
                        -1.3686674435099282,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -2.2902441047566064,
                        6.601201461469724,
                        0.9717150920499783,
                        -2.4405825469508198,
                        0.5321178196948628,
                        -1.8244337021987338,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -7.175068362002635,
                        6.675422271119008,
                        2.407733520011465,
                        -0.9993426967242526,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1.7229307871034958,
                        4.284189778778654,
                        10000003.554359512,
                        -10.973356336291346,
                        -0.46436141906637757,
                        2.451424903945512,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.7289685611777779,
                        -1.8308946338715828,
                        -1.5044428822105846,
                        4.692640923120229,
                        0.1895046228849712,
                        -1.0453117471021265,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.9717150920499783,
                        -2.4405825469508198,
                        -2.0054223619867093,
                        6.255290350519265,
                        0.2526096623056667,
                        -1.3934005588871345,
                        0.0,
                        0.0,
                        -10000000.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -3.3612866353936672,
                        4.205156433963181,
                        -0.46436141906637757,
                        2.451424903945512,
                        10000010.373279463,
                        -16.99855337809098,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -10000000.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.3991881618116,
                        -1.3686674435099282,
                        0.1895046228849712,
                        -1.0453117471021265,
                        -1.3304102563362856,
                        4.426997386700979,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.5321178196948628,
                        -1.8244337021987338,
                        0.2526096623056667,
                        -1.3934005588871345,
                        -1.7734368716962687,
                        5.901187516472405,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.407733520011465,
                        -0.9993426967242526,
                        -7.224938066978001,
                        6.644943442253672,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        11.58941479605369,
                        -10.653610816526951,
                        -7.224938066978001,
                        6.644943442253672,
                        2.407733520011465,
                        -0.9993426967242526,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -4.3349628401868,
                        3.986966065352203,
                        1.4446401120068793,
                        -0.5996056180345515,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -3.8523736320183444,
                        1.5989482976228442,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -7.224938066978001,
                        6.644943442253672,
                        7.238228562540131,
                        -6.65257100102314,
                        -2.421024015573595,
                        1.006970279055664,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.407733520011465,
                        -0.9993426967242526,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.407733520011465,
                        -0.9993426967242526,
                        -2.421024015573595,
                        1.006970279055664,
                        7.188358857564766,
                        -6.683049829888476,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -7.175068362002635,
                        6.675422271119008,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -10000000.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        10000010.422590783,
                        -7.84528649248852,
                        -1.0907747445126652,
                        2.5013196133428393,
                        1.0809513347493516,
                        -2.4925941140824843,
                        2.4286016759142925,
                        -3.079395289523383,
                        -10.412767372774814,
                        7.836567846683939,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -2.4286016759142925,
                        3.079395289523383,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -10000000.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1.0907747445126652,
                        2.5013196133428393,
                        10000010.422590783,
                        -7.84528649248852,
                        -10.412767372774814,
                        7.836567846683939,
                        2.428601675914292,
                        -3.079395289523384,
                        1.0809513347493516,
                        -2.4925941140824843,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -2.428601675914292,
                        3.079395289523384,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1.0809513347493516,
                        -2.4925941140824843,
                        -10.412767372774814,
                        7.836567846683939,
                        10.496968761663704,
                        -7.834876246351111,
                        -2.428601675914292,
                        3.079395289523384,
                        -1.0809513347493516,
                        2.4925941140824843,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.428601675914292,
                        -3.079395289523384,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.4286016759142925,
                        -3.079395289523383,
                        2.428601675914292,
                        -3.079395289523384,
                        -2.428601675914292,
                        3.079395289523384,
                        11.55855071287394,
                        -8.17317835062127,
                        -2.4286016759142925,
                        3.079395289523383,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -11.546745157318384,
                        8.197439395398543,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -10.412767372774814,
                        7.836567846683939,
                        1.0809513347493516,
                        -2.4925941140824843,
                        -1.0809513347493516,
                        2.4925941140824843,
                        -2.4286016759142925,
                        3.079395289523383,
                        10.463114594997036,
                        -7.8386956907955545,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.4286016759142925,
                        -3.079395289523383,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        6.559515660905306,
                        -6.61643249334516,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -6.5300017720164165,
                        6.619904745555312,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        4.313312154270448,
                        -1.6527297604254776,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -4.291089932048226,
                        1.6378059450141607,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1.718112606719134,
                        4.952139130885014,
                        0.7289685611777779,
                        -1.8308946338715828,
                        0.3991881618116,
                        -1.3686674435099282,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        5.1547133280766975,
                        -14.850728807398516,
                        -2.1858127771447613,
                        5.489938889308774,
                        -1.1969660024335829,
                        4.103950312512208,
                        -3.4336493324686748,
                        9.896853765321925,
                        1.4568442159669834,
                        -3.6590442982770908,
                        0.797777840621983,
                        -2.73528291184218,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.7289685611777779,
                        -1.8308946338715828,
                        -1.5044428822105846,
                        4.692640923120229,
                        0.1895046228849712,
                        -1.0453117471021265,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -2.1858127771447613,
                        5.489938889308774,
                        4.522531443410049,
                        -14.077484348000528,
                        -0.5682297537780246,
                        3.1343680144378414,
                        1.4568442159669834,
                        -3.6590442982770908,
                        -3.0066302278661308,
                        9.37824640257761,
                        0.3787251308930535,
                        -2.0890563101756148,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.3991881618116,
                        -1.3686674435099282,
                        0.1895046228849712,
                        -1.0453117471021265,
                        -1.3304102563362856,
                        4.426997386700979,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1.1969660024335829,
                        4.103950312512208,
                        -0.5682297537780246,
                        3.1343680144378414,
                        4.0095486509333895,
                        -13.286160338247655,
                        0.797777840621983,
                        -2.73528291184218,
                        0.3787251308930535,
                        -2.0890563101756148,
                        -2.658825894597104,
                        8.847357595910653,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -1.1451220523783032,
                        3.300600730734862,
                        0.48585754602498915,
                        -1.2202912734754099,
                        0.2660589098474314,
                        -0.9122168510993669,
                        -5.564622525570931,
                        7.133818691956606,
                        1.495155595100418,
                        -1.5059091209875801,
                        2.1394799752143716,
                        -1.6901171050679111,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -3.4336493324686748,
                        9.896853765321925,
                        1.4568442159669834,
                        -3.6590442982770908,
                        0.797777840621983,
                        -2.73528291184218,
                        10.14339391041791,
                        -20.33127287144081,
                        -3.4378573570923905,
                        6.3852446249031,
                        -3.203316725683786,
                        5.337616800172477,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.48585754602498915,
                        -1.2202912734754099,
                        -1.0027111809933547,
                        3.1276451752596324,
                        0.12630483115283336,
                        -0.6967002794435673,
                        1.495155595100418,
                        -1.5059091209875801,
                        -4.908541379197551,
                        7.063422502477067,
                        1.0880240059003325,
                        -1.3823967084607216,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -4.3349628401868,
                        3.986966065352203,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1.4568442159669834,
                        -3.6590442982770908,
                        -3.0066302278661308,
                        9.37824640257761,
                        0.3787251308930535,
                        -2.0890563101756148,
                        -3.4378573570923905,
                        6.3852446249031,
                        13.252845628243836,
                        -23.55627977911405,
                        -3.0376940799530985,
                        4.7677588375674995,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1.4446401120068793,
                        -0.5996056180345515,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.2660589098474314,
                        -0.9122168510993669,
                        0.12630483115283336,
                        -0.6967002794435673,
                        -0.8867184358481344,
                        2.9505937582362023,
                        2.1394799752143716,
                        -1.6901171050679111,
                        1.0880240059003325,
                        -1.3823967084607216,
                        -5.189203684098949,
                        7.085613396831461,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1.4446401120068793,
                        -0.5996056180345515,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.797777840621983,
                        -2.73528291184218,
                        0.3787251308930535,
                        -2.0890563101756148,
                        -2.658825894597104,
                        8.847357595910653,
                        -3.203316725683786,
                        5.337616800172477,
                        -3.0376940799530985,
                        4.7677588375674995,
                        13.03978903174577,
                        -22.888817747097256,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -4.3050410172015825,
                        4.0052533626714055,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -2.2902441047566064,
                        6.601201461469724,
                        0.9717150920499783,
                        -2.4405825469508198,
                        0.5321178196948628,
                        -1.8244337021987338,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.2902441047566064,
                        -6.601201361509958,
                        -0.9717150920499783,
                        2.44058252553087,
                        -0.5321178196948628,
                        1.824433680778784,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.9717150920499783,
                        -2.4405825469508198,
                        -2.0054223619867093,
                        6.255290350519265,
                        0.2526096623056667,
                        -1.3934005588871345,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -0.9717150920499783,
                        2.44058252553087,
                        2.0054223619867093,
                        -6.255290250559498,
                        -0.2526096623056667,
                        1.3934005374671847,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.5321178196948628,
                        -1.8244337021987338,
                        0.2526096623056667,
                        -1.3934005588871345,
                        -1.7734368716962687,
                        5.901187516472405,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -0.5321178196948628,
                        1.824433680778784,
                        -0.2526096623056667,
                        1.3934005374671847,
                        1.7734368716962687,
                        -5.901187416512638,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -3.8523736320183444,
                        1.5989482976228442,
                        2.407733520011465,
                        -0.9993426967242526,
                        -7.175068362002635,
                        6.675422271119008,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        1.4446401120068793,
                        -0.5996056180345515,
                        -4.3050410172015825,
                        4.0052533626714055,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        11.480109379204219,
                        -10.680675553822601,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -10000000.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -2.4286016759142925,
                        3.079395289523383,
                        -2.428601675914292,
                        3.079395289523384,
                        2.428601675914292,
                        -3.079395289523384,
                        -11.546745157318384,
                        8.197439395398543,
                        2.4286016759142925,
                        -3.079395289523383,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        10000011.546745157,
                        -8.197432541942769,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -7.175068362002635,
                        6.675422271119008,
                        0.0,
                        0.0,
                        2.407733520011465,
                        -0.9993426967242526,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -4.291089932048226,
                        1.6378059450141607,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        11.46615829405086,
                        -8.313221446001002,
                        -2.407733520011465,
                        0.9993426902982676,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        2.407733520011465,
                        -0.9993426967242526,
                        0.0,
                        0.0,
                        -7.224938066978001,
                        6.644943442253672,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -6.5300017720164165,
                        6.619904745555312,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -2.407733520011465,
                        0.9993426902982676,
                        13.754939838994417,
                        -13.264848127833126]
            actual = dss.circuit.system_y
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_bus_distances(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [0.0, 0.0, 0.0, 0.762, 0.762, 1.2192, 0.762, 0.85344, 1.2202, 1.3726, 1.40208, 1.55448,
                        0.8129016,
                        0.6096, 1.524, 1.31064]
            actual = dss.circuit.buses_distances
            assert actual == expected

    def test_all_node_distances(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.762, 0.762, 0.762, 0.762, 0.762, 0.762, 1.2192,
                        1.2192, 1.2192, 0.762, 0.762, 0.85344, 0.85344, 1.2202, 1.2202, 1.2202, 1.3726, 1.3726, 1.3726,
                        1.40208, 1.55448, 0.8129016, 0.8129016, 0.8129016, 0.6096, 0.6096, 0.6096, 1.524, 1.524, 1.524,
                        1.31064, 1.31064]
            actual = dss.circuit.nodes_distances
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_node_vmag_by_phase(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [66394.86979179642,
                        2401.7070791741353,
                        2491.569167533136,
                        2466.670172181299,
                        279.4619105506327,
                        2498.515832891544,
                        2449.165950335357,
                        2445.0446146882796,
                        2498.5158301567194,
                        2504.1760259677267,
                        2478.326882914011,
                        2471.2992596093127,
                        2498.5158685371916]
            actual = dss.circuit.nodes_vmag_by_phase(2)
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_node_vmag_pu_by_phase(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [0.9999938073157516,
                        0.9999708379873679,
                        1.0373856703700681,
                        1.027018765320356,
                        1.0084213080290931,
                        1.040277972616208,
                        1.0197307360934018,
                        1.018014783512685,
                        1.0402779714775408,
                        1.042634641363467,
                        1.031872134367998,
                        1.0289461246035285,
                        1.0402779874575714]
            actual = dss.circuit.nodes_vmag_pu_by_phase(2)
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_node_distances_by_phase(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [0.0, 0.0, 0.0, 0.762, 0.762, 1.2192, 0.762, 0.85344, 1.2202, 1.3726, 1.40208, 0.8129016, 0.6096,
                        1.524, 1.31064]
            actual = dss.circuit.nodes_distances_by_phase(3)
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_all_node_names_by_phase(self, dss):
        expected = ['sourcebus.0', '650.0', 'rg60.0', '633.0', '634.0', '671.0', '645.0', '646.0', '692.0', '675.0',
                    '670.0', '632.0', '680.0']
        actual = dss.circuit.nodes_names_by_phase(2)
        assert actual == expected

    def test_y_node_varray(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [57502.68622482564,
                        33189.47561163729,
                        -10.988868660972098,
                        -66394.86888242468,
                        -57491.697356164674,
                        33205.39328374589,
                        2401.562774320432,
                        -0.46690091155609226,
                        -1201.2376790663666,
                        -2079.7175126796437,
                        -1200.3116016198273,
                        2080.1419403490627,
                        2536.356120228099,
                        -0.5793286234950956,
                        -1246.2598772664523,
                        -2157.4877137350218,
                        -1267.5877694598496,
                        2196.935539327288,
                        2426.4262832082777,
                        -109.96557825373392,
                        -1300.0178453618119,
                        -2096.2860825922003,
                        -1120.4252829545628,
                        2128.604896193387,
                        273.12085209243287,
                        -15.65326869615496,
                        -149.22087189563993,
                        -236.28815213487715,
                        -124.73890495654827,
                        242.00752558006553,
                        2350.078784828011,
                        -221.0796482325248,
                        -1338.403130602017,
                        -2109.8005657418066,
                        -1015.4071554561633,
                        2083.115730276637,
                        -1295.6847720988399,
                        -2078.3683079842394,
                        -1296.2419078314438,
                        -2073.161856729391,
                        -1121.7911317832359,
                        2124.3537662394724,
                        -1015.4071496666263,
                        2083.115713199055,
                        2350.0787629180386,
                        -221.07964093085243,
                        2333.500398562236,
                        -229.7550420165211,
                        -1347.9807493549972,
                        -2110.4135775719074,
                        -1013.9682244842037,
                        2078.6483630432763,
                        -1002.1266180672632,
                        2078.753428367298,
                        2332.4291296649008,
                        -217.30934684180974,
                        2407.055227539429,
                        -145.37221180195039,
                        -1312.300335059707,
                        -2102.3729377007444,
                        -1083.00064496635,
                        2116.1077231577606,
                        2433.8502466486893,
                        -107.52285925941754,
                        -1300.7615949301178,
                        -2101.270878230694,
                        -1123.5656240409871,
                        2134.1389201731345,
                        2350.078813428106,
                        -221.07966574213089,
                        -1338.4031538526617,
                        -2109.800593205293,
                        -1015.407159783762,
                        2083.1157650090286,
                        -1122.3895089817488,
                        2129.5620039280107,
                        -1338.4031344410694,
                        -2109.8005600677147,
                        2345.389845277566,
                        -221.599795561614,
                        -1009.5693326805186,
                        2080.532633420145]
            actual = dss.circuit.y_node_varray
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]

    def test_y_node_order(self, dss):
        expected = ['SOURCEBUS.1', 'SOURCEBUS.2', 'SOURCEBUS.3', '650.1', '650.2', '650.3', 'RG60.1', 'RG60.2',
                    'RG60.3', '633.1', '633.2', '633.3', '634.1', '634.2', '634.3', '671.1', '671.2', '671.3', '645.2',
                    '646.2', '646.3', '692.3', '692.1', '675.1', '675.2', '675.3', '611.3', '652.1', '670.1', '670.2',
                    '670.3', '632.1', '632.2', '632.3', '680.1', '680.2', '680.3', '645.3', '692.2', '684.1', '684.3']
        actual = dss.circuit.y_node_order
        assert actual == expected

    def test_y_currents(self, dss):
        if platform.architecture()[0] == "64bit":
            expected = [69802.42815021734,
                        -72191.08720768025,
                        -97420.52952591579,
                        -24355.13240711842,
                        27618.10139734361,
                        96546.21962201368,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        -13.83377137730065,
                        10.726871597154286,
                        -8.96850477084314,
                        -3.417089130031627,
                        -3.124989922944067,
                        -18.470505046170615,
                        -2.6670486287958752,
                        3.8320562835381793,
                        -0.46758569533285765,
                        -1.9484679417919324,
                        3.134634324128733,
                        -1.8835883417462753,
                        -3.314466624037081,
                        -1.321239610087293,
                        -7.10543e-15,
                        7.10543e-15,
                        7.10543e-15,
                        -7.10543e-15,
                        0.25870863043273395,
                        -1.0385283598224362,
                        -0.25870863043273395,
                        1.0385283598224362,
                        -9.011574972518474,
                        4.595357740771433,
                        -3.0888821721242863,
                        -0.88627512815067,
                        -1.9603496363736994,
                        -10.93190933549596,
                        0.02816179626887383,
                        -3.015812588180637,
                        0.0,
                        0.0,
                        0.06464719014596998,
                        -0.043479621379271194,
                        -1.7990236975013154,
                        -0.9604538679363728,
                        -0.05795649932674385,
                        -1.0830078082713541,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0,
                        0.0]
            actual = dss.circuit.y_currents
            assert [round(value, 20) for value in actual] == [round(value, 20) for value in expected]
