# -*- coding: utf-8 -*-
# @Time    : 8/26/2021 08:20 PM
# @Author  : Rodolfo Londero
# @Email   : rodolfpl@gmail.com
# @File    : test_monitors.py
# @Software: PyCharm


import os

import pytest

path = os.path.dirname(os.path.abspath(__file__))


class TestMonitors13Bus:

    @pytest.fixture(scope='function')
    def dss(self, solve_snap_13bus):
        dss = solve_snap_13bus
        dss.text("set mode=daily")
        dss.text("New monitor.m1 element=Transformer.XFM1 terminal=1 mode=0")
        dss.text(f"Set DataPath={path}")
        dss.text("Set maxcontrol=100")
        dss.text("Set maxiterations=100")
        dss.text("Set controlmode=off")
        dss.text("Set mode=daily stepsize=1.0h number=24")
        dss.text(f"batchedit load..* daily=default")

        dss.solution.solve()

        dss.monitors.name = "m1"

        return dss

    # ===================================================================
    # Integer methods
    # ===================================================================
    def test_monitors_first(self, dss):
        expected = 1
        actual = dss.monitors.first()
        assert actual == expected

    def test_monitors_next(self, dss):
        expected = 0
        actual = dss.monitors.next()
        assert actual == expected

    def test_monitors_count(self, dss):
        expected = 1
        actual = dss.monitors.count
        assert actual == expected

    def test_monitors_reset(self, dss):
        expected = 0
        actual = dss.monitors.reset()
        assert actual == expected

    def test_monitors_reset_all(self, dss):
        expected = 0
        actual = dss.monitors.reset_all()
        assert actual == expected

    def test_monitors_sample(self, dss):
        expected = 0
        actual = dss.monitors.sample()
        assert actual == expected

    def test_monitors_sample_all(self, dss):
        expected = 0
        actual = dss.monitors.sample_all()
        assert actual == expected

    def test_monitors_save(self, dss):
        expected = 0
        actual = dss.monitors.save()
        assert actual == expected

    def test_monitors_save_all(self, dss):
        expected = 0
        actual = dss.monitors.save_all()
        assert actual == expected

    def test_monitors_process(self, dss):
        expected = 0
        actual = dss.monitors.process()
        assert actual == expected

    def test_monitors_process_all(self, dss):
        expected = 0
        actual = dss.monitors.process_all()
        assert actual == expected

    def test_monitors_file_version(self, dss):
        # TODO: file_version returns different values each time
        expected = 0
        actual = dss.monitors.file_version
        assert type(actual) == type(expected)

    def test_monitors_sample_count(self, dss):
        expected = 24
        actual = dss.monitors.sample_count
        assert actual == expected

    def test_monitors_record_size(self, dss):
        expected = 16
        actual = dss.monitors.record_size
        assert actual == expected

    def test_monitors_num_channels(self, dss):
        expected = 16
        actual = dss.monitors.num_channels
        assert actual == expected

    def test_monitors_read_terminal(self, dss):
        expected = 1
        actual = dss.monitors.terminal
        assert actual == expected

    def test_monitors_write_terminal(self, dss):
        expected = 2
        dss.monitors.terminal = expected
        actual = dss.monitors.terminal
        assert actual == expected

    # ===================================================================
    # String methods
    # ===================================================================
    def test_monitors_file_name(self, dss):
        expected = os.path.join(path, "IEEE13Nodeckt_Mon_m1_1.csv")
        actual = dss.monitors.file_name
        assert expected.lower() == actual.lower()

    def test_monitors_read_name(self, dss):
        expected = "m1"
        actual = dss.monitors.name
        assert expected.lower() == actual.lower()

    def test_monitors_write_name(self, dss):
        dss.text("New monitor.m2 element=Transformer.XFM1 terminal=2 mode=0")
        expected = "m2"
        dss.monitors.name = expected
        actual = dss.monitors.name
        assert expected.lower() == actual.lower()

    def test_monitors_read_element(self, dss):
        expected = "transformer.xfm1"
        actual = dss.monitors.element
        assert actual == expected

    def test_monitors_write_element(self, dss):
        expected = "load.671"
        dss.monitors.element = expected
        actual = dss.monitors.element
        assert actual == expected

    # ===================================================================
    # Variant methods
    # ===================================================================
    def test_monitors_all_names(self, dss):
        dss.text("New monitor.m2 element=Transformer.XFM1 terminal=2 mode=0")
        expected = ["m1", "m2"]
        actual = dss.monitors.names
        assert actual == expected

    def test_monitors_byte_stream(self, dss):
        expected = [236,
                    170,
                    0,
                    0,
                    1,
                    0,
                    0,
                    0,
                    16,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    128,
                    63,
                    0,
                    0,
                    0,
                    0,
                    52,
                    140,
                    26,
                    69,
                    252,
                    201,
                    221,
                    191,
                    206,
                    251,
                    26,
                    69,
                    237,
                    135,
                    242,
                    194,
                    37,
                    4,
                    26,
                    69,
                    172,
                    197,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    176,
                    249,
                    87,
                    66,
                    138,
                    189,
                    18,
                    194,
                    117,
                    206,
                    37,
                    66,
                    114,
                    114,
                    30,
                    195,
                    167,
                    224,
                    38,
                    66,
                    171,
                    102,
                    162,
                    66,
                    95,
                    66,
                    76,
                    65,
                    216,
                    167,
                    22,
                    67,
                    0,
                    0,
                    0,
                    64,
                    0,
                    0,
                    0,
                    0,
                    33,
                    247,
                    26,
                    69,
                    43,
                    154,
                    204,
                    191,
                    172,
                    28,
                    27,
                    69,
                    90,
                    92,
                    242,
                    194,
                    68,
                    148,
                    26,
                    69,
                    58,
                    248,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    176,
                    201,
                    70,
                    66,
                    137,
                    14,
                    18,
                    194,
                    203,
                    240,
                    24,
                    66,
                    110,
                    86,
                    30,
                    195,
                    35,
                    128,
                    25,
                    66,
                    205,
                    166,
                    162,
                    66,
                    168,
                    66,
                    59,
                    65,
                    15,
                    149,
                    22,
                    67,
                    0,
                    0,
                    64,
                    64,
                    0,
                    0,
                    0,
                    0,
                    3,
                    26,
                    27,
                    69,
                    6,
                    244,
                    198,
                    191,
                    105,
                    39,
                    27,
                    69,
                    26,
                    78,
                    242,
                    194,
                    138,
                    195,
                    26,
                    69,
                    233,
                    8,
                    237,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    87,
                    41,
                    65,
                    66,
                    82,
                    213,
                    17,
                    194,
                    122,
                    185,
                    20,
                    66,
                    64,
                    77,
                    30,
                    195,
                    62,
                    29,
                    21,
                    66,
                    213,
                    187,
                    162,
                    66,
                    15,
                    184,
                    53,
                    65,
                    103,
                    137,
                    22,
                    67,
                    0,
                    0,
                    128,
                    64,
                    0,
                    0,
                    0,
                    0,
                    53,
                    78,
                    27,
                    69,
                    198,
                    128,
                    190,
                    191,
                    145,
                    55,
                    27,
                    69,
                    166,
                    56,
                    242,
                    194,
                    22,
                    10,
                    27,
                    69,
                    223,
                    33,
                    237,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    67,
                    190,
                    56,
                    66,
                    119,
                    127,
                    17,
                    194,
                    170,
                    100,
                    14,
                    66,
                    118,
                    63,
                    30,
                    195,
                    168,
                    145,
                    14,
                    66,
                    97,
                    219,
                    162,
                    66,
                    176,
                    118,
                    45,
                    65,
                    250,
                    126,
                    22,
                    67,
                    0,
                    0,
                    160,
                    64,
                    0,
                    0,
                    0,
                    0,
                    95,
                    84,
                    27,
                    69,
                    0,
                    129,
                    189,
                    191,
                    126,
                    57,
                    27,
                    69,
                    27,
                    54,
                    242,
                    194,
                    113,
                    18,
                    27,
                    69,
                    215,
                    36,
                    237,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    134,
                    191,
                    55,
                    66,
                    70,
                    117,
                    17,
                    194,
                    127,
                    164,
                    13,
                    66,
                    206,
                    61,
                    30,
                    195,
                    24,
                    203,
                    13,
                    66,
                    33,
                    223,
                    162,
                    66,
                    58,
                    127,
                    44,
                    65,
                    94,
                    125,
                    22,
                    67,
                    0,
                    0,
                    192,
                    64,
                    0,
                    0,
                    0,
                    0,
                    197,
                    38,
                    27,
                    69,
                    225,
                    223,
                    196,
                    191,
                    84,
                    43,
                    27,
                    69,
                    232,
                    72,
                    242,
                    194,
                    239,
                    212,
                    26,
                    69,
                    19,
                    15,
                    237,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    29,
                    25,
                    63,
                    66,
                    121,
                    192,
                    17,
                    194,
                    68,
                    46,
                    19,
                    66,
                    226,
                    73,
                    30,
                    195,
                    16,
                    128,
                    19,
                    66,
                    133,
                    195,
                    162,
                    66,
                    96,
                    176,
                    51,
                    65,
                    36,
                    130,
                    22,
                    67,
                    0,
                    0,
                    224,
                    64,
                    0,
                    0,
                    0,
                    0,
                    247,
                    181,
                    26,
                    69,
                    190,
                    14,
                    215,
                    191,
                    139,
                    8,
                    27,
                    69,
                    10,
                    119,
                    242,
                    194,
                    185,
                    60,
                    26,
                    69,
                    125,
                    217,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    89,
                    65,
                    81,
                    66,
                    59,
                    121,
                    18,
                    194,
                    171,
                    204,
                    32,
                    66,
                    143,
                    103,
                    30,
                    195,
                    83,
                    160,
                    33,
                    66,
                    190,
                    127,
                    162,
                    66,
                    121,
                    148,
                    69,
                    65,
                    110,
                    151,
                    22,
                    67,
                    0,
                    0,
                    0,
                    65,
                    0,
                    0,
                    0,
                    0,
                    38,
                    247,
                    25,
                    69,
                    56,
                    150,
                    245,
                    191,
                    105,
                    206,
                    26,
                    69,
                    4,
                    196,
                    242,
                    194,
                    140,
                    59,
                    25,
                    69,
                    243,
                    127,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    231,
                    219,
                    111,
                    66,
                    17,
                    176,
                    19,
                    194,
                    38,
                    147,
                    55,
                    66,
                    37,
                    153,
                    30,
                    195,
                    180,
                    129,
                    57,
                    66,
                    1,
                    14,
                    162,
                    66,
                    73,
                    30,
                    100,
                    65,
                    120,
                    199,
                    22,
                    67,
                    0,
                    0,
                    16,
                    65,
                    0,
                    0,
                    0,
                    0,
                    60,
                    66,
                    25,
                    69,
                    69,
                    27,
                    9,
                    192,
                    253,
                    151,
                    26,
                    69,
                    245,
                    11,
                    243,
                    194,
                    162,
                    72,
                    24,
                    69,
                    143,
                    44,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    184,
                    92,
                    134,
                    66,
                    155,
                    212,
                    20,
                    194,
                    56,
                    225,
                    76,
                    66,
                    132,
                    199,
                    30,
                    195,
                    1,
                    20,
                    80,
                    66,
                    104,
                    163,
                    161,
                    66,
                    224,
                    170,
                    128,
                    65,
                    157,
                    241,
                    22,
                    67,
                    0,
                    0,
                    32,
                    65,
                    0,
                    0,
                    0,
                    0,
                    158,
                    217,
                    24,
                    69,
                    158,
                    81,
                    17,
                    192,
                    222,
                    120,
                    26,
                    69,
                    17,
                    53,
                    243,
                    194,
                    118,
                    188,
                    23,
                    69,
                    246,
                    252,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    97,
                    173,
                    142,
                    66,
                    236,
                    124,
                    21,
                    194,
                    140,
                    17,
                    89,
                    66,
                    7,
                    226,
                    30,
                    195,
                    253,
                    27,
                    93,
                    66,
                    67,
                    102,
                    161,
                    66,
                    136,
                    46,
                    137,
                    65,
                    25,
                    11,
                    23,
                    67,
                    0,
                    0,
                    48,
                    65,
                    0,
                    0,
                    0,
                    0,
                    38,
                    85,
                    24,
                    69,
                    185,
                    165,
                    27,
                    192,
                    208,
                    81,
                    26,
                    69,
                    160,
                    104,
                    243,
                    194,
                    91,
                    11,
                    23,
                    69,
                    95,
                    193,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    31,
                    44,
                    153,
                    66,
                    0,
                    81,
                    22,
                    194,
                    186,
                    93,
                    104,
                    66,
                    78,
                    3,
                    31,
                    195,
                    160,
                    148,
                    109,
                    66,
                    111,
                    25,
                    161,
                    66,
                    169,
                    7,
                    148,
                    65,
                    231,
                    42,
                    23,
                    67,
                    0,
                    0,
                    64,
                    65,
                    0,
                    0,
                    0,
                    0,
                    133,
                    231,
                    23,
                    69,
                    179,
                    35,
                    36,
                    192,
                    209,
                    49,
                    26,
                    69,
                    221,
                    146,
                    243,
                    194,
                    9,
                    121,
                    22,
                    69,
                    155,
                    144,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    186,
                    212,
                    161,
                    66,
                    201,
                    255,
                    22,
                    194,
                    182,
                    232,
                    116,
                    66,
                    149,
                    30,
                    31,
                    195,
                    148,
                    49,
                    123,
                    66,
                    73,
                    218,
                    160,
                    66,
                    248,
                    17,
                    157,
                    65,
                    21,
                    70,
                    23,
                    67,
                    0,
                    0,
                    80,
                    65,
                    0,
                    0,
                    0,
                    0,
                    98,
                    240,
                    23,
                    69,
                    52,
                    118,
                    35,
                    192,
                    130,
                    52,
                    26,
                    69,
                    97,
                    143,
                    243,
                    194,
                    179,
                    132,
                    22,
                    69,
                    130,
                    148,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    59,
                    34,
                    161,
                    66,
                    106,
                    242,
                    22,
                    194,
                    250,
                    225,
                    115,
                    66,
                    85,
                    28,
                    31,
                    195,
                    71,
                    27,
                    122,
                    66,
                    40,
                    223,
                    160,
                    66,
                    87,
                    92,
                    156,
                    65,
                    153,
                    71,
                    23,
                    67,
                    0,
                    0,
                    96,
                    65,
                    0,
                    0,
                    0,
                    0,
                    151,
                    251,
                    23,
                    69,
                    215,
                    152,
                    34,
                    192,
                    203,
                    55,
                    26,
                    69,
                    14,
                    139,
                    243,
                    194,
                    158,
                    147,
                    22,
                    69,
                    120,
                    153,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    250,
                    63,
                    160,
                    66,
                    186,
                    224,
                    22,
                    194,
                    228,
                    153,
                    114,
                    66,
                    139,
                    25,
                    31,
                    195,
                    225,
                    183,
                    120,
                    66,
                    141,
                    229,
                    160,
                    66,
                    122,
                    112,
                    155,
                    65,
                    156,
                    69,
                    23,
                    67,
                    0,
                    0,
                    112,
                    65,
                    0,
                    0,
                    0,
                    0,
                    189,
                    229,
                    23,
                    69,
                    227,
                    70,
                    36,
                    192,
                    74,
                    49,
                    26,
                    69,
                    143,
                    147,
                    243,
                    194,
                    169,
                    118,
                    22,
                    69,
                    208,
                    143,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    188,
                    248,
                    161,
                    66,
                    142,
                    2,
                    23,
                    194,
                    52,
                    29,
                    117,
                    66,
                    6,
                    31,
                    31,
                    195,
                    55,
                    106,
                    123,
                    66,
                    70,
                    217,
                    160,
                    66,
                    25,
                    55,
                    157,
                    65,
                    81,
                    70,
                    23,
                    67,
                    0,
                    0,
                    128,
                    65,
                    0,
                    0,
                    0,
                    0,
                    0,
                    209,
                    23,
                    69,
                    158,
                    224,
                    37,
                    192,
                    69,
                    43,
                    26,
                    69,
                    128,
                    155,
                    243,
                    194,
                    2,
                    91,
                    22,
                    69,
                    165,
                    134,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    74,
                    155,
                    163,
                    66,
                    138,
                    35,
                    23,
                    194,
                    107,
                    121,
                    119,
                    66,
                    40,
                    36,
                    31,
                    195,
                    228,
                    252,
                    125,
                    66,
                    96,
                    205,
                    160,
                    66,
                    149,
                    238,
                    158,
                    65,
                    134,
                    75,
                    23,
                    67,
                    0,
                    0,
                    136,
                    65,
                    0,
                    0,
                    0,
                    0,
                    190,
                    206,
                    23,
                    69,
                    98,
                    13,
                    38,
                    192,
                    160,
                    42,
                    26,
                    69,
                    92,
                    156,
                    243,
                    194,
                    250,
                    87,
                    22,
                    69,
                    165,
                    133,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    226,
                    200,
                    163,
                    66,
                    48,
                    39,
                    23,
                    194,
                    193,
                    186,
                    119,
                    66,
                    180,
                    36,
                    31,
                    195,
                    26,
                    69,
                    126,
                    66,
                    13,
                    204,
                    160,
                    66,
                    243,
                    30,
                    159,
                    65,
                    146,
                    76,
                    23,
                    67,
                    0,
                    0,
                    144,
                    65,
                    0,
                    0,
                    0,
                    0,
                    1,
                    45,
                    24,
                    69,
                    54,
                    196,
                    30,
                    192,
                    26,
                    70,
                    26,
                    69,
                    27,
                    120,
                    243,
                    194,
                    167,
                    213,
                    22,
                    69,
                    110,
                    175,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    205,
                    88,
                    156,
                    66,
                    90,
                    145,
                    22,
                    194,
                    64,
                    247,
                    108,
                    66,
                    73,
                    13,
                    31,
                    195,
                    105,
                    147,
                    114,
                    66,
                    38,
                    2,
                    161,
                    66,
                    125,
                    87,
                    151,
                    65,
                    203,
                    54,
                    23,
                    67,
                    0,
                    0,
                    152,
                    65,
                    0,
                    0,
                    0,
                    0,
                    248,
                    93,
                    24,
                    69,
                    16,
                    248,
                    26,
                    192,
                    125,
                    84,
                    26,
                    69,
                    36,
                    101,
                    243,
                    194,
                    248,
                    22,
                    23,
                    69,
                    71,
                    197,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    11,
                    122,
                    152,
                    66,
                    123,
                    67,
                    22,
                    194,
                    9,
                    87,
                    103,
                    66,
                    10,
                    1,
                    31,
                    195,
                    219,
                    127,
                    108,
                    66,
                    82,
                    30,
                    161,
                    66,
                    188,
                    82,
                    147,
                    65,
                    176,
                    44,
                    23,
                    67,
                    0,
                    0,
                    160,
                    65,
                    0,
                    0,
                    0,
                    0,
                    238,
                    144,
                    24,
                    69,
                    154,
                    0,
                    23,
                    192,
                    120,
                    99,
                    26,
                    69,
                    92,
                    81,
                    243,
                    194,
                    19,
                    91,
                    23,
                    69,
                    34,
                    220,
                    235,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    70,
                    113,
                    148,
                    66,
                    243,
                    241,
                    21,
                    194,
                    71,
                    120,
                    97,
                    66,
                    69,
                    244,
                    30,
                    195,
                    54,
                    42,
                    102,
                    66,
                    213,
                    59,
                    161,
                    66,
                    36,
                    36,
                    143,
                    65,
                    97,
                    32,
                    23,
                    67,
                    0,
                    0,
                    168,
                    65,
                    0,
                    0,
                    0,
                    0,
                    77,
                    226,
                    24,
                    69,
                    11,
                    166,
                    16,
                    192,
                    139,
                    123,
                    26,
                    69,
                    148,
                    49,
                    243,
                    194,
                    221,
                    199,
                    23,
                    69,
                    214,
                    0,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    174,
                    253,
                    141,
                    66,
                    162,
                    111,
                    21,
                    194,
                    200,
                    11,
                    88,
                    66,
                    194,
                    223,
                    30,
                    195,
                    66,
                    12,
                    92,
                    66,
                    19,
                    107,
                    161,
                    66,
                    164,
                    126,
                    136,
                    65,
                    109,
                    14,
                    23,
                    67,
                    0,
                    0,
                    176,
                    65,
                    0,
                    0,
                    0,
                    0,
                    95,
                    226,
                    24,
                    69,
                    143,
                    162,
                    16,
                    192,
                    122,
                    123,
                    26,
                    69,
                    160,
                    49,
                    243,
                    194,
                    36,
                    200,
                    23,
                    69,
                    234,
                    0,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    146,
                    251,
                    141,
                    66,
                    246,
                    110,
                    21,
                    194,
                    208,
                    12,
                    88,
                    66,
                    205,
                    223,
                    30,
                    195,
                    252,
                    5,
                    92,
                    66,
                    81,
                    107,
                    161,
                    66,
                    81,
                    120,
                    136,
                    65,
                    247,
                    9,
                    23,
                    67,
                    0,
                    0,
                    184,
                    65,
                    0,
                    0,
                    0,
                    0,
                    228,
                    74,
                    25,
                    69,
                    206,
                    109,
                    8,
                    192,
                    154,
                    154,
                    26,
                    69,
                    131,
                    8,
                    243,
                    194,
                    36,
                    84,
                    24,
                    69,
                    123,
                    48,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    179,
                    172,
                    133,
                    66,
                    217,
                    198,
                    20,
                    194,
                    21,
                    221,
                    75,
                    66,
                    72,
                    197,
                    30,
                    195,
                    45,
                    2,
                    79,
                    66,
                    98,
                    168,
                    161,
                    66,
                    228,
                    240,
                    127,
                    65,
                    212,
                    241,
                    22,
                    67,
                    0,
                    0,
                    192,
                    65,
                    0,
                    0,
                    0,
                    0,
                    126,
                    229,
                    25,
                    69,
                    219,
                    102,
                    248,
                    191,
                    22,
                    201,
                    26,
                    69,
                    15,
                    203,
                    242,
                    194,
                    179,
                    35,
                    25,
                    69,
                    181,
                    119,
                    236,
                    66,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    0,
                    121,
                    175,
                    114,
                    66,
                    216,
                    204,
                    19,
                    194,
                    30,
                    170,
                    57,
                    66,
                    173,
                    157,
                    30,
                    195,
                    195,
                    184,
                    59,
                    66,
                    120,
                    3,
                    162,
                    66,
                    224,
                    246,
                    102,
                    65,
                    11,
                    206,
                    22,
                    67]

        dss.monitors.save()

        actual = dss.monitors.byte_stream
        assert actual == expected

    def test_monitors_header(self, dss):
        expected = [' V1', ' VAngle1', ' V2', ' VAngle2', ' V3', ' VAngle3', ' V4', ' VAngle4',
                    ' I1', ' IAngle1', ' I2', ' IAngle2', ' I3', ' IAngle3', ' I4', ' IAngle4\x00']
        actual = dss.monitors.header
        assert actual == expected

    def test_monitors_dbl_hour(self, dss):
        expected = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24]
        actual = dss.monitors.dbl_hour
        assert actual == expected

    def test_monitors_dbl_freq(self, dss):
        expected = [0.0]
        actual = dss.monitors.dbl_freq
        assert actual == expected

    def test_monitors_channel(self, dss):
        expected = [2472.76,
                    2479.45,
                    2481.63,
                    2484.89,
                    2485.27,
                    2482.42,
                    2475.37,
                    2463.45,
                    2452.14,
                    2445.6,
                    2437.32,
                    2430.47,
                    2431.02,
                    2431.72,
                    2430.36,
                    2429.06,
                    2428.92,
                    2434.81,
                    2437.87,
                    2441.06,
                    2446.14,
                    2446.15,
                    2452.68,
                    2462.34]

        actual = dss.monitors.channel(1)
        assert [round(value, 2) for value in actual] == [round(value, 2) for value in expected]
